// ********************************************************************
// >>>>>>>>>>>>>>>>>>>>>>>>> COPYRIGHT NOTICE <<<<<<<<<<<<<<<<<<<<<<<<<
// ********************************************************************
// File name    : top.v
// Module name  : top
// Author       : STEP
// Description  : 
// Web          : www.stepfpga.com
// 
// --------------------------------------------------------------------
// Code Revision History : 
// --------------------------------------------------------------------
// Version: |Mod. Date:   |Changes Made:
// V1.0     |2017/03/02   |Initial ver
// --------------------------------------------------------------------
// Module Function:进过按键消抖后控制led显示翻转
 
module top(clk,rst,key,din,led,wsig,ssig,dout,clk_flash);
 
    input             clk;
    input             rst;
    input       [1:0] key;
    input             din;	 
	 output  reg [2:0] led;
	 output  reg       wsig;
	 output            ssig;
	 output            dout;
	 output            clk_flash;
 	 
	 wire  [1:0] key_pulse;
	 
	 assign clk_flash = ~clk;
	 
	 reg write_clk_reg;
	 reg read_clk_reg;
	 
	 reg [7:0] writecnt;
	 reg [7:0] readcnt;
	 
	 reg write_ssig=1;
	 reg read_ssig =1;
	 
	 reg write_dout=1;
	 reg read_dout =1;
	 
	 assign ssig = write_ssig & read_ssig;
	 assign dout = write_dout & read_dout;
	 
 
	always @(posedge clk  or  negedge rst) begin
		if (!rst) begin              // 初始化
			write_clk_reg <= 1'b1;
			read_clk_reg <= 1'b1;
		end
		else if (key_pulse[0]) begin           
				write_clk_reg <= 1'b0; // 开始写flash
			end
		else if (key_pulse[1]) begin
				read_clk_reg <= 1'b0;  // 开始读flash
			end
		else begin
				write_clk_reg <= 1'b1;
				read_clk_reg <= 1'b1;
			end
	end    
	
	always @(posedge clk or negedge write_clk_reg) begin  //写处理
		if(!write_clk_reg) begin
				writecnt <= 0;
				write_ssig <= 1;  // 写WRITE ENABLE 8’b0000 0110 命令之前要把S#拉高
				write_dout <=1;
			end
		else begin
				case(writecnt)
					8'h00: begin writecnt<= writecnt+ 8'h01; end   // 等待一个cycle，让S#置高电平
					8'h01: begin writecnt<= writecnt+ 8'h01; end   // 等待一个cycle，让S#置高电平
					8'h02: begin write_ssig<=0; write_dout<= 1'b0; writecnt<= writecnt+ 8'h01; end // S#拉低；WRITE ENABLE bit7
					8'h03: begin write_dout<= 1'b0; writecnt<= writecnt+ 8'h01; end // WRITE ENABLE bit6
					8'h04: begin write_dout<= 1'b0; writecnt<= writecnt+ 8'h01; end // WRITE ENABLE bit5
					8'h05: begin write_dout<= 1'b0; writecnt<= writecnt+ 8'h01; end // WRITE ENABLE bit4
					8'h06: begin write_dout<= 1'b0; writecnt<= writecnt+ 8'h01; end // WRITE ENABLE bit3
					8'h07: begin write_dout<= 1'b1; writecnt<= writecnt+ 8'h01; end // WRITE ENABLE bit2
					8'h08: begin write_dout<= 1'b1; writecnt<= writecnt+ 8'h01; end // WRITE ENABLE bit1
					8'h09: begin write_dout<= 1'b0; writecnt<= writecnt+ 8'h01; end // WRITE ENABLE bit0
					8'h0A: begin write_ssig<=1; writecnt<= writecnt+ 8'h01; end // S#拉高
					8'h0B: begin writecnt<= writecnt+ 8'h01; end // S#拉高保持一个cycle，后面开始写WRITE STATUS REGISTER
					8'h0C: begin write_ssig<=0; write_dout<=0; writecnt<= writecnt+ 8'h01; end // S#低；WriTE STATUS REGISTER bit 7
					8'h0D: begin write_dout<=0; writecnt<= writecnt+ 8'h01; end // S#低；WriTE STATUS REGISTER bit 6
					8'h0E: begin write_dout<=0; writecnt<= writecnt+ 8'h01; end // S#低；WriTE STATUS REGISTER bit 5
					8'h0F: begin write_dout<=0; writecnt<= writecnt+ 8'h01; end // S#低；WriTE STATUS REGISTER bit 4
					8'h10: begin write_dout<=0; writecnt<= writecnt+ 8'h01; end // S#低；WriTE STATUS REGISTER bit 3
					8'h11: begin write_dout<=0; writecnt<= writecnt+ 8'h01; end // S#低；WriTE STATUS REGISTER bit 2
					8'h12: begin write_dout<=0; writecnt<= writecnt+ 8'h01; end // S#低；WriTE STATUS REGISTER bit 1
					8'h13: begin write_dout<=1; writecnt<= writecnt+ 8'h01; end // S#低；WriTE STATUS REGISTER bit 0
					8'h14: begin write_dout<=0; writecnt<= writecnt+ 8'h01; end // 写STATUS REGISTER的值 bit7
					8'h15: begin write_dout<=0; writecnt<= writecnt+ 8'h01; end // 写STATUS REGISTER的值 bit6
					8'h16: begin write_dout<=0; writecnt<= writecnt+ 8'h01; end // 写STATUS REGISTER的值 bit5
					8'h17: begin write_dout<=0; writecnt<= writecnt+ 8'h01; end // 写STATUS REGISTER的值 bit4
					8'h18: begin write_dout<=0; writecnt<= writecnt+ 8'h01; end // 写STATUS REGISTER的值 bit3
					8'h19: begin write_dout<=1; writecnt<= writecnt+ 8'h01; end // 写STATUS REGISTER的值 bit2
					8'h1A: begin write_dout<=0; writecnt<= writecnt+ 8'h01; end // 写STATUS REGISTER的值 bit1
					8'h1B: begin write_dout<=0; writecnt<= writecnt+ 8'h01; end // 写STATUS REGISTER的值 bit0
					8'h1C: begin write_dout<=1; write_ssig<=1; writecnt<= writecnt+ 8'h01; end // S#拉高
					default: begin write_dout<=1; write_ssig<=1; end
				endcase
			end
	end
	
	
	always @(posedge clk or negedge read_clk_reg) begin    //读处理，先写command，再读取
		if(!read_clk_reg) begin
				readcnt<=0;
				read_ssig <=1;
				read_dout <=1;
			end
		else begin 
				case(readcnt)
					8'h00: begin readcnt<=readcnt+8'h01; end // 等待一个cycle，让S#置高电平
					8'h01: begin readcnt<=readcnt+8'h01; end // 等待一个cycle，让S#置高电平
					8'h02: begin read_ssig<=0; read_dout<=0; readcnt<=readcnt+8'h01; end // S#拉低，写READ STATUS REGISTER命令bit7
					8'h03: begin read_dout<=0; readcnt<=readcnt+8'h01; end // 写bit6
					8'h04: begin read_dout<=0; readcnt<=readcnt+8'h01; end // 写bit5
					8'h05: begin read_dout<=0; readcnt<=readcnt+8'h01; end // 写bit4
					8'h06: begin read_dout<=0; readcnt<=readcnt+8'h01; end // 写bit3
					8'h07: begin read_dout<=1; readcnt<=readcnt+8'h01; end // 写bit2
					8'h08: begin read_dout<=0; readcnt<=readcnt+8'h01; end // 写bit1
					8'h09: begin read_dout<=1; readcnt<=readcnt+8'h01; end // 写bit0
					8'h0A: begin readcnt<=readcnt+8'h01; end // 写完READ STATUS REGISTER，DQ1开始输出数据，读STATUS REGISTER bit7
					8'h0B: begin readcnt<=readcnt+8'h01; end // bit6
					8'h0C: begin readcnt<=readcnt+8'h01; end // bit5
					8'h0D: begin readcnt<=readcnt+8'h01; end // bit4
					8'h0E: begin readcnt<=readcnt+8'h01; end // bit3
					8'h0F: begin readcnt<=readcnt+8'h01; end // bit2
					8'h10: begin readcnt<=readcnt+8'h01; end // bit1
					8'h11: begin readcnt<=readcnt+8'h01; end // bit0
					8'h12: begin read_ssig<=1;readcnt<=readcnt+8'h01; end // 读完，S#拉高
					default: begin read_dout<=1; read_ssig<=1; end
				endcase
			end
	end
	
	always @(negedge clk)begin
		/*if(din)begin
			led = 3'b000;
		end*/
		case(readcnt)
			
			/*
			8'h0B: begin led<= led | {3{din}}; end
			8'h0C: begin led<= led | {3{din}}; end
			8'h0D: begin led<= led | {3{din}}; end
			
			8'h0E: begin led[2]<=din; end
			8'h0F: begin led[1]<=din; end
			8'h10: begin led[0]<=din; end
			
			8'h11: begin led[2]<=din; end
			8'h12: begin led[1]<=din; end*/
		
			
			8'h0E: begin led[2]<=din; end
			8'h0F: begin led[1]<=din; end
			8'h10: begin led[0]<=din; end
			
			default:;
		endcase
	end
	
	//例化消抖module，这里没有传递参数N，采用了默认的N=1     
    Key_debounce  u1 (                               
                  .clk (clk),
                  .rst (rst),
                  .key (key),
                  .key_pulse (key_pulse)
                  );
 endmodule